/* ***************************************************************************+
 * ITX package (cnrg.itx) for telephony application programming.              *
 * Copyright (c) 1999  Cornell University, Ithaca NY                          *
 *                                                                            *
 * This program is free software; you can redistribute it and/or modify       *
 * it under the terms of the GNU General Public Liense as published by        *
 * the Free Software Foundation, either version 2 of the License, or (at      * 
 * your option) any later version.                                            *
 *                                                                            *
 * The ITX package is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY, without even the implied warranty of MERCHANTABILITY *
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License   *
 * for more details.                                                          * 
 *                                                                            *
 * A copy of the license is distributed with this package.  Look in the docs  *
 * directory, filename GPL.                                                   *
 *                                                                            * 
 * Contact information:                                                       *
 * Donna Bergmark                                                             *
 * 484 Rhodes Hall                                                            *
 * Cornell University                                                         *
 * Ithaca, NY 14853-3801                                                      *
 * bergmark@cs.cornell.edu                                                    *
 ******************************************************************************/

package cnrg.itx.datax;

/**
 * Inteface to abstract a destination. Any device that wants to behave like
 * a destination for the channel has to implement this interface.
 */
public interface Destination extends Statistics, Properties
{
	/**
	 * Method to implement the writing to the destination.
	 * @param data the byte array to be written to the destination
	 * @exception DataException thrown if the write method fails
	 */
	public void write (byte[] data) throws DataException;
	
	/**
	 * Method to mute the destination device.
	 * @param mode true if the destination has to be muted, false otherwise
	 * @return boolean true if mute succeeds, false otherwise
	 */
	public boolean mute (boolean mode);
	
	/**
	 * Method to close the destination device.
	 */
	public void close() ;
}