/* ***************************************************************************+
 * ITX package (cnrg.itx) for telephony application programming.              *
 * Copyright (c) 1999  Cornell University, Ithaca NY                          *
 *                                                                            *
 * This program is free software; you can redistribute it and/or modify       *
 * it under the terms of the GNU General Public Liense as published by        *
 * the Free Software Foundation, either version 2 of the License, or (at      * 
 * your option) any later version.                                            *
 *                                                                            *
 * The ITX package is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY, without even the implied warranty of MERCHANTABILITY *
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License   *
 * for more details.                                                          * 
 *                                                                            *
 * A copy of the license is distributed with this package.  Look in the docs  *
 * directory, filename GPL.                                                   *
 *                                                                            * 
 * Contact information:                                                       *
 * Donna Bergmark                                                             *
 * 484 Rhodes Hall                                                            *
 * Cornell University                                                         *
 * Ithaca, NY 14853-3801                                                      *
 * bergmark@cs.cornell.edu                                                    *
 ******************************************************************************/


package cnrg.itx.datax;

/**
 * Interface that exports the setting and getting of properties on a device. 
 */
public interface Properties
{
   /**
    * Interface to return a collection of properties supported by the device.
    * @exception DataException thrown when the properties in the device are incompatible
    */ 
   public PropertiesCollection getProperties() throws DataException;

   /**
    * Interface to set the given properties collection into the device. Works under the assumption
    * that the properties collection was generated LOCALLY and is a request for options.
    * @exception DataException thrown when trying to set incompatible properties collections
    */
   public void setProperties(PropertiesCollection pc) throws DataException;

   /**
    * Interface to set the given properties collection into the device. WOrks under the 
    * assumption that this is the properties collection of the peer.
    */
   public void setPeerProperties(PropertiesCollection pc) throws DataException;
}
