/* ***************************************************************************+
 * ITX package (cnrg.itx) for telephony application programming.              *
 * Copyright (c) 1999  Cornell University, Ithaca NY                          *
 *                                                                            *
 * This program is free software; you can redistribute it and/or modify       *
 * it under the terms of the GNU General Public Liense as published by        *
 * the Free Software Foundation, either version 2 of the License, or (at      * 
 * your option) any later version.                                            *
 *                                                                            *
 * The ITX package is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY, without even the implied warranty of MERCHANTABILITY *
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License   *
 * for more details.                                                          * 
 *                                                                            *
 * A copy of the license is distributed with this package.  Look in the docs  *
 * directory, filename GPL.                                                   *
 *                                                                            * 
 * Contact information:                                                       *
 * Donna Bergmark                                                             *
 * 484 Rhodes Hall                                                            *
 * Cornell University                                                         *
 * Ithaca, NY 14853-3801                                                      *
 * bergmark@cs.cornell.edu                                                    *
 ******************************************************************************/

package cnrg.itx.signal.SignalEvent;

import cnrg.itx.datax.Connection;
import cnrg.itx.signal.*;

/** This class represents an event generated when a dial sequence failed and the call needs to
 *  be aborted.
 */
public class AbortSignalEvent extends SignalEvent { 
	
	/**  Reason why the call is being aborted  **/
	private String myAbortReason;
	/**  Data exchange connection that is being aborted  **/
	private Connection myOldConn;
	
	/**
	 * constructor for an AbortSignalEvent
	 * @param p_conn the connection given to signaling if the onInvite function accepts with accept(Connection);
	 * @param p_abortReason the reason for the abortion
	 */
	public AbortSignalEvent(InvitePacket ip, Connection c, String reason){
		super(ip);
		myOldConn = c;
		myAbortReason = reason;
	}
	
	/**
	 * @return the old Connection object
	 */
	public Connection getConnection() {
		return myOldConn;
	}
	
	/**
	 * @return the abort reason
	 */
	public String getAbortReason() {
		return myAbortReason;
	}
}
